#!/bin/bash

set -e

function note_step {
    echo
    echo
    echo '==============================================================================='
    echo "[$(date)] STEP $1:" $2
    echo '==============================================================================='
    echo
}

note_step "0 - A" "Installing Python dependencies"
apt-get update
apt-get install -y --no-install-recommends python3-pandas python3-numpy python3-yaml python3-seaborn
#pip install -r requirements.txt

note_step "0 - B" "Compiling the Rust-based RTA tool"

cd rta-tool
cargo build --release
cd ..

note_step "0 - C" "Running the library test suite"

cd response-time-analysis
cargo test
cd ..

note_step "0 - D" "Running the library test suite"

cd rta-tool
cargo test
cd ..

note_step "0 - E" "Generating the documentation"

cd rta-tool
cargo doc
cd ..

note_step 1 "Generating synthetic workloads"

python3 tsgen.py

note_step 2 "Running the RTAs"

./run-rta.sh

note_step 3 "Crunching the data"

python3 eval.py

note_step 4 "Copying the results"

./copy-results.sh

note_step 5 "Plotting the results"

python3 plot.py

note_step 6 "[DONE] All steps completed --- please check the generated plots."

